@@ -15,26 +15,6 @@ ALLOWED_CARD_TYPES = {"notebook": 1, "voila": 2}
 MAX_FILE_SIZE = 5000000
 
 
-def approve_card(card_id: int) -> None:
-    """
-    Approve and un-hide a specific card in the Gallery.
-
-    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/admin/cards/approve_card.ipynb) to test this function or build upon it.
-
-    :param card_id: the id of the card to be approved
-    :type card_id: int
-    :return: Nothing
-    :rtype: None
-    """
-
-    adaboard.request_adaboard(
-        path=f"cards/{card_id}/approval",
-        method=requests.put,
-    )
-
-    return None
-
-
 def _create_keywords_if_not_exist(keywords: list[str]) -> dict[str, int]:
     """Helper function to create keywords if they do not exist in the Gallery.
 
@@ -70,9 +50,10 @@ def create_card(  # noqa: C901
     acl_list_edit: list[str] = [],
     aux_files: list[dict[str, str]] = [{"source": "", "target": ""}],
     coauthors: list[str] = [],
-    picture_id: int = 0,
     include_kernel: bool = True,
     keywords: list[str] = [],
+    picture_id: int = 0,
+    reviewers: list[str] = [],
     source: str = "lab",
 ) -> int:
     """
@@ -104,12 +85,14 @@ def create_card(  # noqa: C901
     :type aux_files: dict, optional
     :param coauthors: the other card authors, defaults to []
     :type coauthors: list, optional
-    :param picture_id: the ID of the picture to be used for the card, defaults to 0
-    :type picture_id: int, optional
     :param include_kernel: whether to link the Notebook with the kernel used during its development, defaults to True
     :type include_kernel: bool, optional
     :param keywords: the keywords related to the card, defaults to [] (tip: must exist in the Gallery)
     :type keywords: list, optional
+    :param picture_id: the ID of the picture to be used for the card, defaults to 0
+    :type picture_id: int, optional
+    :param reviewers: the list of users who will review the card, defaults to []
+    :type reviewers: list, optional
     :param source: the source of the card files, defaults to "lab" (tip: must be "lab" or "local")
     :type source: str, optional
     :return: the ID of the card
@@ -317,6 +300,7 @@ def create_card(  # noqa: C901
         "coauthors": coauthors,
         "acls": acl_config,
         "content": content,
+        "reviewers": reviewers,
     }
 
     response = adaboard.request_adaboard(
@@ -337,8 +321,9 @@ def create_card_group(
     acl_type_edit: str = "public",
     acl_list_edit: list[str] = [],
     coauthors: list[str] = [],
-    picture_id: int = 0,
     keywords: list[str] = [],
+    picture_id: int = 0,
+    reviewers: list[str] = [],
 ) -> int:
     """
     Create a card group in the AdaLab Gallery.
@@ -362,10 +347,12 @@ def create_card_group(
     :type acl_list_edit: list, optional
     :param coauthors: the other group authors, defaults to []
     :type coauthors: list, optional
-    :param picture_id: the ID of the picture to be used for the card group, defaults to 0
-    :type picture_id: int, optional
     :param keywords: the keywords related to the card group, defaults to [] (tip: must exist in the Gallery)
     :type keywords: list, optional
+    :param picture_id: the ID of the picture to be used for the card group, defaults to 0
+    :type picture_id: int, optional
+    :param reviewers: the list of users who will review the card group, defaults to []
+    :type reviewers: list, optional
     :return: the ID of the card group
     :rtype: int
     """
@@ -415,6 +402,7 @@ def create_card_group(
         "keywords": keywords_id,
         "coauthors": coauthors,
         "acls": acl_config,
+        "reviewers": reviewers,
     }
 
     response = adaboard.request_adaboard(
@@ -438,8 +426,9 @@ def create_url_card(
     acl_type_edit: str = "public",
     acl_list_edit: list[str] = [],
     coauthors: list[str] = [],
-    picture_id: int = 0,
     keywords: list[str] = [],
+    picture_id: int = 0,
+    reviewers: list[str] = [],
 ) -> int:
     """Create a URL-type card in the AdaLab Gallery.
 
@@ -467,10 +456,12 @@ def create_url_card(
     :type acl_list_edit: list, optional
     :param coauthors: the other card authors, defaults to []
     :type coauthors: list, optional
-    :param picture_id: the ID of the picture to be used for the card, defaults to 0
-    :type picture_id: int, optional
     :param keywords: the keywords related to the card, defaults to [] (tip: must exist in the Gallery)
     :type keywords: list, optional
+    :param picture_id: the ID of the picture to be used for the card, defaults to 0
+    :type picture_id: int, optional
+    :param reviewers: the list of users who will review the card, defaults to []
+    :type reviewers: list, optional
     :return: the ID of the card
     :rtype: int
     """
@@ -534,6 +525,7 @@ def create_url_card(
         "keywords": keywords_id,
         "coauthors": coauthors,
         "acls": acl_config,
+        "reviewers": reviewers,
     }
 
     response = adaboard.request_adaboard(
@@ -552,7 +544,7 @@ def create_url_subtype(
     """
     Create a new URL subtype in the AdaLab Gallery.
 
-    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/cards/create_url_subtype.ipynb) to test this function or build upon it.
+    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/admin/cards/create_url_subtype.ipynb) to test this function or build upon it.
 
     :param name: the name of the URL subtype
     :type name: str, optional
@@ -637,26 +629,6 @@ def delete_url_subtype(url_subtype_id: int) -> None:
     return None
 
 
-def deregister_as_reviewer(card_id: int) -> None:
-    """
-    De-register invoker as card reviewer.
-
-    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/admin/cards/deregister_as_reviewer.ipynb) to test this function or build upon it.
-
-    :param card_id: the id of the card for which the user will stop being the reviewer
-    :type card_id: int
-    :return: Nothing
-    :rtype: None
-    """
-
-    adaboard.request_adaboard(
-        path=f"cards/{card_id}/reviewer",
-        method=requests.delete,
-    )
-
-    return None
-
-
 def edit_card(  # noqa: C901
     card_id: int,
     name: str = None,
@@ -671,10 +643,11 @@ def edit_card(  # noqa: C901
     acl_list_edit: list[str] = None,
     aux_files: dict[str, str] = None,
     coauthors: list[str] = None,
-    picture_id: int = None,
-    keywords: list[str] = None,
-    source: str = "lab",
     keep_aux_files: bool = True,
+    keywords: list[str] = None,
+    picture_id: int = None,
+    reviewers: list[str] = None,
+    source: str = "lab",
 ) -> None:
     """
     Edit a card in the AdaLab Gallery. Note that unspecified fields will not be changed.
@@ -707,14 +680,16 @@ def edit_card(  # noqa: C901
     :type aux_files: dict, optional
     :param coauthors: the other card authors
     :type coauthors: list, optional
-    :param picture_id: the ID of the picture to be used for the card
-    :type picture_id: int, optional
-    :param keywords: the keywords related to the card, defaults to [] (tip: must exist in the Gallery)
-    :type keywords: list, optional
-    :param source: the source of the card files, defaults to "lab" (tip: must be "lab" or "local")
-    :type source: str, optional
     :param keep_aux_files: whether to keep the existing auxiliary files or not, defaults to True
     :type keep_aux_files: bool, optional
+    :param keywords: the keywords related to the card, defaults to [] (tip: must exist in the Gallery)
+    :type keywords: list, optional
+    :param picture_id: the ID of the picture to be used for the card
+    :type picture_id: int, optional
+    :param reviewers: the list of users who will review the card
+    :type reviewers: list, optional
+    :param source: the source of the card files, defaults to "lab" (tip: must be "lab" or "local")
+    :type source: str, optional
     :return: nothing
     :rtype: None
     """
@@ -759,6 +734,7 @@ def edit_card(  # noqa: C901
         "coauthors": coauthors,
         "acls": None,
         "content": None,
+        "reviewers": reviewers,
     }
     # Fetch old configuration
     old_config = adaboard.request_adaboard(
@@ -835,6 +811,17 @@ def edit_card(  # noqa: C901
         },
     ]
     card_config["acls"] = acl_config
+    # Author, coauthors and reviewers need to be re-formatted for the PUT request
+    old_author = old_config["author"]["user_id"]
+    old_config["author"] = old_author
+    old_coauthors = [
+        coauthor["user_id"] for coauthor in old_config["coauthors"]
+    ]
+    old_config["coauthors"] = old_coauthors
+    old_reviewers = [
+        reviewer["user_id"] for reviewer in old_config["reviewers"]
+    ]
+    old_config["reviewers"] = old_reviewers
     # Update content
     # First, the main Notebook
     if notebook_file:
@@ -998,8 +985,9 @@ def edit_card_group(
     acl_type_edit: str = None,
     acl_list_edit: list[str] = None,
     coauthors: list[str] = None,
-    picture_id: int = None,
     keywords: list[str] = None,
+    picture_id: int = None,
+    reviewers: list[str] = None,
 ) -> None:
     """
     Edit a card group in the AdaLab Gallery. Note that the unspecified fields will not be modified.
@@ -1024,10 +1012,12 @@ def edit_card_group(
     :type acl_list_edit: list, optional
     :param coauthors: the other authors of the card group
     :type coauthors: list, optional
-    :param picture_id: the ID of the new picture to be used for the card group
-    :type picture_id: int, optional
     :param keywords: the keywords related to the card group
     :type keywords: list, optional
+    :param picture_id: the ID of the new picture to be used for the card group
+    :type picture_id: int, optional
+    :param reviewers: the list of users who will review the card group
+    :type reviewers: list, optional
     """
 
     validate_acl("card", True, acl_type_view, acl_type_edit)
@@ -1050,6 +1040,7 @@ def edit_card_group(
         "keywords": keywords_id,
         "coauthors": coauthors,
         "acls": None,
+        "reviewers": reviewers,
     }
     # Fetch old configuration
     old_config = adaboard.request_adaboard(
@@ -1114,7 +1105,19 @@ def edit_card_group(
         },
     ]
     card_group_config["acls"] = acl_config
+    # Author, coauthors and reviewers need to be re-formatted for the PUT request
+    old_author = old_config["author"]["user_id"]
+    old_config["author"] = old_author
+    old_coauthors = [
+        coauthor["user_id"] for coauthor in old_config["coauthors"]
+    ]
+    old_config["coauthors"] = old_coauthors
+    old_reviewers = [
+        reviewer["user_id"] for reviewer in old_config["reviewers"]
+    ]
+    old_config["reviewers"] = old_reviewers
 
+    # Update the card group cards
     if cards_id:
         old_config["card_group_cards"] = []
         for card_id in cards_id:
@@ -1149,8 +1152,9 @@ def edit_url_card(
     acl_type_edit: str = None,
     acl_list_edit: list[str] = None,
     coauthors: list[str] = None,
-    picture_id: int = None,
     keywords: list[str] = None,
+    picture_id: int = None,
+    reviewers: list[str] = None,
 ) -> None:
     """
     Edit a card of the URL type in the AdaLab Gallery. Note that unspecified fields will not be changed.
@@ -1182,10 +1186,12 @@ def edit_url_card(
     :param coauthors: _description_, defaults to None
     :param coauthors: the other card authors
     :type coauthors: list, optional
-    :param picture_id: the ID of the picture to be used for the card
-    :type picture_id: int, optional
     :param keywords: the keywords related to the card, defaults to [] (tip: must exist in the Gallery)
     :type keywords: list, optional
+    :param picture_id: the ID of the picture to be used for the card
+    :type picture_id: int, optional
+    :param reviewers: the list of users who will review the card
+    :type reviewers: list, optional
     :return: nothing
     :rtype: None
     """
@@ -1225,6 +1231,7 @@ def edit_url_card(
         "keywords": keywords_id,
         "coauthors": coauthors,
         "acls": None,
+        "reviewers": reviewers,
     }
     # Fetch old configuration
     old_config = adaboard.request_adaboard(
@@ -1301,6 +1308,17 @@ def edit_url_card(
         },
     ]
     card_config["acls"] = acl_config
+    # Author, coauthors and reviewers need to be re-formatted for the PUT request
+    old_author = old_config["author"]["user_id"]
+    old_config["author"] = old_author
+    old_coauthors = [
+        coauthor["user_id"] for coauthor in old_config["coauthors"]
+    ]
+    old_config["coauthors"] = old_coauthors
+    old_reviewers = [
+        reviewer["user_id"] for reviewer in old_config["reviewers"]
+    ]
+    old_config["reviewers"] = old_reviewers
 
     # Build request payload combining new and old options
     payload = {
@@ -1660,25 +1678,6 @@ def launch_card(
     return card_path
 
 
-def register_as_reviewer(card_id: int) -> None:
-    """
-    Register invoker as card reviewer.
-
-    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/admin/cards/deregister_as_reviewer.ipynb) to test this function or build upon it.
-
-    :param card_id: the id of the card for which the user will become reviewer
-    :type card_id: int
-    :return: Nothing
-    :rtype: None
-    """
-
-    adaboard.request_adaboard(
-        path=f"cards/{card_id}/reviewer", method=requests.put
-    )
-
-    return None
-
-
 def set_card_visibility(card_id: int, new_status: str) -> None:
     """
     Set the visibility status of a specific card in the Gallery.
@@ -1688,7 +1687,7 @@ def set_card_visibility(card_id: int, new_status: str) -> None:
     :param card_id: the id of the card whose visibility is to be set
     :type card_id: int
     :param new_status: visibility to be set
-    :type new_status: bool
+    :type new_status: str
     :return: Nothing
     :rtype: None
     """
@@ -1725,3 +1724,22 @@ def toggle_card_favorite(card_id: int) -> str:
     ).json()
 
     return response["message"]
+
+
+def toggle_card_review(card_id: int) -> None:
+    """
+    Toggle the review status of a card in the Gallery.
+
+    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/cards/toggle_card_review.ipynb) to test this function or build upon it.
+
+    :param card_id: the id of the card whose status is to be toggled
+    :type card_id: int
+    :return: nothing
+    :rtype: None
+    """
+
+    adaboard.request_adaboard(
+        path=f"cards/{card_id}/review", method=requests.put
+    )
+
+    return None
