@@ -12,6 +12,7 @@ from .harbor import (
     get_project_repositories,
     get_project_stats,
     get_projects,
+    get_publish_logs,
     get_repositories_by_author,
     get_repositories_by_type,
     get_repository_tags,
@@ -33,6 +34,7 @@ __all__ = [
     "get_project_repositories",
     "get_projects",
     "get_project_stats",
+    "get_publish_logs",
     "get_repositories_by_author",
     "get_repositories_by_type",
     "get_repository_tags",
