@@ -13,6 +13,7 @@ ALLOWED_OCI_PROJECT_TYPES = [
 ALLOWED_METADATA_TYPES = ["app", "base", "internal", "kernel", "lab_app"]
 
 ALLOWED_OCI_SOURCE_TYPES_OPERATIONS = {"lab": "push", "registry": "copy"}
+ALLOWED_PUBLISH_LOG_SOURCES = ["publish", "system"]
 HARBOR_HOST = "container-registry"
 HARBOR_PORT = "80"
 
@@ -357,7 +358,7 @@ def get_image_id(project_name: str, repository_name: str, tag: str) -> int:
     :type: repository_name: str
     :param tag: the tag of the specific image
     :type tag: str
-    :return: image metadata ID
+    :return: image ID
     :rtype: int
     """
     # Make sure image options are valid
@@ -367,7 +368,7 @@ def get_image_id(project_name: str, repository_name: str, tag: str) -> int:
 
     # Get all images from the repository, page by page
     all_images = adaboard.get_all_pages(
-        api_url=f"registry/{project_name}/{repository_name}/images/"
+        api_url=f"registry/{project_name}/{repository_name}/images/v2/"
     )
     # Find the specific image and return its ID
     for image in all_images:
@@ -465,7 +466,7 @@ def get_project_repositories(project_name: str) -> list[str]:
 
     # Get all repositories from the project, page by page
     all_images = adaboard.get_all_pages(
-        api_url=f"registry/{project_name}/repositories/"
+        api_url=f"registry/{project_name}/v2/repositories/"
     )
 
     return [x["name"] for x in all_images]
@@ -490,7 +491,7 @@ def get_project_stats(project_name: str) -> list[dict[str, str | int]]:
 
     # Get all repositories from the project, page by page
     all_images = adaboard.get_all_pages(
-        api_url=f"registry/{project_name}/repositories/"
+        api_url=f"registry/{project_name}/v2/repositories/"
     )
 
     # Compile and return statistics
@@ -526,6 +527,68 @@ def get_projects(include_archived: bool = False) -> list[str]:
     return list(set(projects))
 
 
+def get_publish_logs(
+    metadata_id: int, source: str = "publish"
+) -> list[dict[str, str]]:
+    """
+    Get the publish or system logs of an image publishing process.
+
+    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/harbor/get_publish_logs.ipynb) to test this function or build upon it.
+
+    :param metadata_id: the metadata ID of the image
+    :type metadata_id: int
+    :param source: the source of the logs, either "build" or "publish", defaults to "publish"
+    :type source: str, optional
+    :return: build logs
+    :rtype: list
+    """
+
+    assert (
+        source in ALLOWED_PUBLISH_LOG_SOURCES
+    ), f"Log source must be one of {', '.join([x for x in ALLOWED_PUBLISH_LOG_SOURCES])}"
+
+    # Get run ID of the publishing process
+    try:
+        mdata = adaboard.request_adaboard(
+            path="registry/metadata?get_archived=false",
+        ).json()
+        mdata_archived = adaboard.request_adaboard(
+            path="registry/metadata?get_archived=true",
+        ).json()
+        mdata.extend(mdata_archived)
+        publish_id = [
+            x["run_id"] for x in mdata if x["metadata_id"] == metadata_id
+        ][0]
+    except IndexError:
+        raise ValueError(
+            f"Could not find metadata with ID {metadata_id} in the registry."
+        )
+
+    response_raw = adaboard.request_adaboard(
+        path=f"script_runner/runs/{publish_id}/logs/"
+    ).json()
+    response = response_raw["log"]
+
+    if source == "publish":
+        logs = [
+            {
+                "time": res["timestamp"],
+                "input": res["input"],
+                "output": res["output"].replace(res["input"], ""),
+            }
+            for res in response
+            if res["type"] == "command"
+        ]
+    elif source == "system":
+        logs = [
+            {"time": res["timestamp"], "message": res["output"]}
+            for res in response
+            if res["type"] == "system"
+        ]
+
+    return logs
+
+
 def get_repositories_by_author(
     author_id: str,
     include_archived: bool = False,
@@ -613,7 +676,7 @@ def get_repository_tags(project_name: str, repository_name: str) -> list[str]:
 
     # Get all images from the repository, page by page
     all_images = adaboard.get_all_pages(
-        api_url=f"registry/{project_name}/{repository_name}/images/"
+        api_url=f"registry/{project_name}/{repository_name}/images/v2/"
     )
 
     return [response["tags"][0] for response in all_images]
