@@ -1,4 +1,5 @@
 import base64
+from datetime import datetime
 from typing import Optional
 
 import requests
@@ -7,48 +8,54 @@ from adalib_auth import config
 from .. import adaboard
 
 ALLOWED_GIT_TYPES = ["ssh", "https", "public"]
+ALLOWED_BUILD_LOG_SOURCES = ["build", "system"]
+ALLOWED_LAB_LOG_SOURCES = ["user", "system"]
 ALLOWED_OCI_PROJECT_TYPES = ["apps", "kernels", "base_images"]
 HARBOR_HOST = "container-registry"
 HARBOR_PORT = "80"
+LOG_SOURCES = {"user": False, "system": True}
 
 
 def build_image_from_git(
     git_url: str,
-    git_type: str,
+    file_path: str,
     image_name: str,
     image_tag: str,
     image_type: str,
-    project_name: str = "",
+    branch: str = "",
     build_args: dict[str, str] = {},
-    timeout: int = 3600,
-    is_docker: bool = False,
+    commit: str = "",
     git_creds: dict[str, str] = {},
+    project_name: str = "",
+    timeout: int = 3600,
 ) -> int:
     """
-    Build a container image from a Containerfile in a Git repository.
+    Build a container image from a container file in a Git repository.
 
     Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/lab/build_image_from_git.ipynb) to test this function or build upon it.
 
-    :param git_url: full URL of Git repository containing the Containerfile
+    :param git_url: full URL of Git repository containing the container file
     :type git_url: str
-    :param git_type: type of Git repository, either "SSH" or "HTTPS" or "public"
-    :type git_type: str
+    :param file_path: path to the container file in the Git repository
+    :type file_path: str
     :param image_name: the name of the target image
     :type image_name: str
     :param image_tag: the tag of the target image
     :type image_tag: str
     :param image_type: the type of the image
     :type image_type: str
-    :param project_name: the name of the project to link the image to, defaults to image_name
-    :type project_name: str, optional
+    :param branch: the branch of the Git repository to use, defaults to default
+    :type branch: str, optional
     :param build_args: additional build arguments to pass build process, defaults to {}
     :type build_args: dict, optional
-    :param timeout: the maximum time (in seconds) to wait for the build to complete, defaults to 3600
-    :type timeout: int, optional
-    :param is_docker: whether the original file is a Dockerfile, defaults to False
-    :type is_docker: bool, optional
+    :param commit: the commit hash of the Git repository to use, defaults to ""
+    :type commit: str, optional
     :param git_creds: credentials to authenticate to the Git repository. For SSH, use keys "PRIVATE_KEY_FILE" and "PASSWORD". For HTTPS, use keys "USERNAME" and "PASSWORD".
     :type git_creds: dict, optional
+    :param project_name: the name of the project to link the image to, defaults to image_name
+    :type project_name: str, optional
+    :param timeout: the maximum time (in seconds) to wait for the build to complete, defaults to 3600
+    :type timeout: int, optional
     :return: the run ID of the build process
     :rtype: int
     """
@@ -58,15 +65,32 @@ def build_image_from_git(
     assert (
         image_type in ALLOWED_OCI_PROJECT_TYPES
     ), f"Can only build images that are of the type {', '.join([x for x in ALLOWED_OCI_PROJECT_TYPES])}"
-    assert (
-        git_type.lower() in ALLOWED_GIT_TYPES
-    ), f"Git type must be one of {', '.join([x for x in ALLOWED_GIT_TYPES])}"
+    # Check that only branch or commit is specified
+    assert not (
+        bool(branch) and bool(commit)
+    ), "Cannot specify both 'branch' and 'commit'"
+    # Check that the file path is valid
+    file_name = file_path.split("/")[-1]
+    assert ("dockerfile" in file_name.lower()) or (
+        "containerfile" in file_name.lower()
+    ), "The file path must point at a 'Dockerfile' or 'Containerfile' type of file."
+    # Figure out git type
+    if git_creds:
+        if list(git_creds.keys()) == ["PRIVATE_KEY_FILE", "PASSWORD"]:
+            git_type = "ssh"
+        elif list(git_creds.keys()) == ["USERNAME", "PASSWORD"]:
+            git_type = "https"
+        else:
+            raise KeyError(
+                "Dictionary with git credentials has the wrong keys. For SSH, use 'PRIVATE_KEY_FILE' and 'PASSWORD'. For HTTPS, use 'USERNAME' and 'PASSWORD'."
+            )
+    else:
+        git_type = "public"
 
     # Build request payload with the specified parameters
     # Prepare build arguments
     build_args = ";;".join([f"{k}={v}" for k, v in build_args.items()])
-    # Set correct Containerfile type
-    file_type = "Dockerfile" if is_docker else "Containerfile"
+
     # Collect parameters into payload
     payload = {
         "script": "builder",
@@ -110,8 +134,8 @@ def build_image_from_git(
                 "value": build_args,
             },
             "IMAGE_CONTAINERFILE_LOCATION": {
-                "real_value": file_type,
-                "value": file_type,
+                "real_value": file_path,
+                "value": file_path,
             },
             "NETWORK_HOST": {
                 "real_value": adalib_config.NETWORK_HOST,
@@ -122,10 +146,6 @@ def build_image_from_git(
 
     # Add Git credentials to payload based on Git type
     if git_type.lower() == "ssh":
-        assert list(git_creds.keys()) == [
-            "PRIVATE_KEY_FILE",
-            "PASSWORD",
-        ], "For SSH, please provide the keys 'PRIVATE_KEY_FILE' and 'PASSWORD' in the 'git_creds' dictionary."
         git_url = git_url.replace("https://", "git@").replace(".com/", ".com:")
         git_config = [
             {
@@ -141,10 +161,6 @@ def build_image_from_git(
             },
         ]
     elif git_type.lower() == "https":
-        assert list(git_creds.keys()) == [
-            "USERNAME",
-            "PASSWORD",
-        ], "For HTTPS, please provide the keys 'USERNAME' and 'PASSWORD' in the 'git_creds' dictionary."
         git_config = [
             {
                 "GIT_URL": {"real_value": git_url, "value": git_url},
@@ -165,6 +181,13 @@ def build_image_from_git(
             },
         ]
 
+    # Use specific branch or commit
+    if branch:
+        git_config[0]["GIT_BRANCH"] = {"real_value": branch, "value": branch}
+    elif commit:
+        git_config[0]["GIT_COMMIT"] = {"real_value": commit, "value": commit}
+
+    # Add Git configuration to payload
     for item in git_config:
         payload["config"].update(item)
 
@@ -179,22 +202,21 @@ def build_image_from_git(
 
 
 def build_image_from_lab(
-    source_repository: str,
+    file_path: str,
     image_name: str,
     image_tag: str,
     image_type: str,
     project_name: str = "",
     build_args: dict[str, str] = {},
     timeout: int = 3600,
-    is_docker: bool = False,
 ) -> int:
     """
-    Build a container image from a Containerfile in the Lab.
+    Build a container image from a container file in the Lab.
 
     Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/lab/build_image_from_lab.ipynb) to test this function or build upon it.
 
-    :param source_repository: Lab directory containing the Containerfile
-    :type source_repository: str
+    :param file_path: path to the container file in the Lab
+    :type file_path: str
     :param image_name: the name of the target image
     :type image_name: str
     :param image_tag: the tag of the target image
@@ -207,8 +229,6 @@ def build_image_from_lab(
     :type build_args: dict, optional
     :param timeout: the maximum time (in seconds) to wait for the build to complete, defaults to 3600
     :type timeout: int, optional
-    :param is_docker: whether the original file is a Dockerfile, defaults to False
-    :type is_docker: bool, optional
     :return: the run ID of the build process
     :rtype: int
     """
@@ -219,9 +239,17 @@ def build_image_from_lab(
         image_type in ALLOWED_OCI_PROJECT_TYPES
     ), f"Can only build images that are of the type {', '.join([x for x in ALLOWED_OCI_PROJECT_TYPES])}"
 
+    # Get the context and file name from the full path
+    ctxt_path = file_path.split("/")[0]
+    file_name = file_path.split("/")[-1]
+
+    assert ("dockerfile" in file_name.lower()) or (
+        "containerfile" in file_name.lower()
+    ), "The file path must point at a 'Dockerfile' or 'Containerfile' type of file."
+
     # Build request payload with the specified parameters
     build_args = ";;".join([f"{k}={v}" for k, v in build_args.items()])
-    file_type = "Dockerfile" if is_docker else "Containerfile"
+
     payload = {
         "script": "builder",
         "user_id": adaboard.get_user()["user_id"],
@@ -232,8 +260,8 @@ def build_image_from_lab(
         "start_podman_sidecar": False,
         "config": {
             "GIT_LOCATION": {
-                "real_value": source_repository,
-                "value": source_repository,
+                "real_value": ctxt_path,
+                "value": ctxt_path,
             },
             "HARBOR_CURATED_PROJECT": {
                 "real_value": image_type,
@@ -268,8 +296,8 @@ def build_image_from_lab(
                 "value": build_args,
             },
             "IMAGE_CONTAINERFILE_LOCATION": {
-                "real_value": file_type,
-                "value": file_type,
+                "real_value": file_name,
+                "value": file_name,
             },
             "NETWORK_HOST": {
                 "real_value": adalib_config.NETWORK_HOST,
@@ -361,6 +389,51 @@ def get_available_kernels() -> (
     return [x for x in response if x["type"] == "kernel"]
 
 
+def get_build_logs(
+    build_id: int, source: str = "build"
+) -> list[dict[str, str]]:
+    """
+    Get the build or system logs of an image build process.
+
+    Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/lab/get_build_logs.ipynb) to test this function or build upon it.
+
+    :param build_id: the run ID of the build process
+    :type build_id: int
+    :param source: the source of the logs, either "build" or "system", defaults to "build"
+    :type source: str, optional
+    :return: build logs
+    :rtype: list
+    """
+
+    assert (
+        source in ALLOWED_BUILD_LOG_SOURCES
+    ), f"Log source must be one of {', '.join([x for x in ALLOWED_BUILD_LOG_SOURCES])}"
+
+    response_raw = adaboard.request_adaboard(
+        path=f"script_runner/runs/{build_id}/logs/"
+    ).json()
+    response = response_raw["log"]
+
+    if source == "build":
+        logs = [
+            {
+                "time": res["timestamp"],
+                "input": res["input"],
+                "output": res["output"].replace(res["input"], ""),
+            }
+            for res in response
+            if res["type"] == "command"
+        ]
+    elif source == "system":
+        logs = [
+            {"time": res["timestamp"], "message": res["output"]}
+            for res in response
+            if res["type"] == "system"
+        ]
+
+    return logs
+
+
 def get_build_status(
     build_id: int,
 ) -> str:
@@ -484,25 +557,54 @@ def get_lab_images() -> list[dict[str, str]]:
     return response
 
 
-def get_lab_logs(since: int = 3600, system: bool = False) -> str:
+def get_lab_logs(
+    from_date: str = "", to_date: str = "", source: str = "user"
+) -> list[dict[str, str]]:
     """
     Get the logs of the user's Lab environment.
 
     Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/lab/get_lab_logs.ipynb) to test this function or build upon it.
 
-    :param since: number of seconds to go back in the logs, defaults to 3600
-    :type since: int, optional
-    :param system: whether to fetch system logs, defaults to False
-    :type system: bool, optional
+    :param from_date: the start date for the logs, in ISO format (YYYY-MM-DDTHH:mm:ss+HH:mm), defaults to ""
+    :type from_date: str, optional
+    :param to_date: the end date for the logs, in ISO format (YYYY-MM-DDTHH:mm:ss+HH:mm), defaults to ""
+    :type to_date: str, optional
+    :param source: the source of the logs, either "user" or "system", defaults to "user"
+    :type source: str, optional
     :return: Lab logs
-    :rtype: str
+    :rtype: list
     """
 
+    # Check that the specified source is valid
+    assert (
+        source in ALLOWED_LAB_LOG_SOURCES
+    ), f"Log source must be one of {', '.join([x for x in ALLOWED_LAB_LOG_SOURCES])}"
+
+    params = f"?&system={LOG_SOURCES[source]}"
+    # Check that the date format is correct
+    if from_date:
+        try:
+            datetime.fromisoformat(from_date)
+        except ValueError:
+            raise AssertionError(
+                "Date must be in ISO format (YYYY-MM-DDTHH:mm:ss+HH:mm)."
+            )
+        params += f"&start_dt={from_date}"
+
+    if to_date:
+        try:
+            datetime.fromisoformat(to_date)
+        except ValueError:
+            raise AssertionError(
+                "Date must be in ISO format (YYYY-MM-DDTHH:mm:ss+HH:mm)."
+            )
+        params += f"&end_dt={to_date}"
+
     response = adaboard.request_adaboard(
-        path=f"jupyter/server/logs?since_seconds={since}&system={system}"
+        path=f"jupyter/server/v2/logs{params}"
     ).json()
 
-    return response["logs"]
+    return response
 
 
 def get_lab_status(metrics: bool = True, options: bool = True) -> dict[
@@ -568,10 +670,9 @@ def install_kernel(
 
 def move_file(old_path: str, new_path: str) -> None:
     """
-    Move a file or folder in the user's Lab environment.
+    Move a file or folder in the user's Lab environment.  Note that this operation can also be used to rename a file or folder.
 
     Use the [example Jupyter Notebook](https://github.com/adamatics/adalib_example_notebooks/blob/main/user/lab/move_file.ipynb) to test this function or build upon it.
-    Note that this operation can also be used to rename a file or folder.
 
     :param old_path: path to the file to move, including file name and extension, relative to the user's home directory
     :type old_path: str
